package com.zego.ktvchorus.rtc;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.google.gson.Gson;
import com.zego.ktvchorus.model.ErrorCodeConstants;
import com.zego.ktvchorus.rtc.callbacks.IPostSDKRequestCallback;

import org.jetbrains.annotations.NotNull;
import org.json.JSONObject;

public class APIBase {
    private static final String TAG = "APIBase";

    private static Gson mGson = new Gson();
    private static final Handler okHandler = new Handler(Looper.getMainLooper());

    public static JsonBuilder builder() {
        return new JsonBuilder();
    }

    public static Gson getGson() {
        return mGson;
    }

    public static <T> void postSDKRequest(@NotNull String url, @NotNull final String reqJson, final Class<T> classType, final IPostSDKRequestCallback<T> reqCallback) {

        if (!ZGKTVCopyrightedMusicManager.getInstance().isInitialized()) {
            if (reqCallback != null) {
                okHandler.post(() -> reqCallback.onResponse(ErrorCodeConstants.ERROR_KTV_SDK_NOT_INITIALIZED, "", null));
            }
            return;
        }

        ZGKTVCopyrightedMusicManager.getInstance().sendExtendedRequest(url, reqJson, (errorCode, url1, result) -> {
            Log.d(TAG, "--url--" + url1 + "--reqJson---" + reqJson + "----result-" + result);
            Log.d(TAG, "--errorCode--" + errorCode + "--url---" + url1 + "----result-" + result);
            if (errorCode == 0) {
                T bean = null;
                try {
                    JSONObject jsonObject = new JSONObject(result);
                    String requestId = jsonObject.getString("request_id");
                    int retcode = jsonObject.getInt("code");
                    String retmsg = jsonObject.getString("message");
                    String dataJson = "";
                    try {
                        dataJson = jsonObject.getJSONObject("data").toString();
                    } catch (Exception jsonException) {
                        dataJson = "";
                    }

                    Log.d(TAG, "--requestId--" + requestId + "--retcode---" + retcode + "----retmsg-" + retmsg);


                    if (reqCallback != null) {
                        bean = APIBase.getGson().fromJson(dataJson, classType);

                        T finalBean = bean;
                        okHandler.post(new Runnable() {
                            @Override
                            public void run() {
                                reqCallback.onResponse(retcode, retmsg, finalBean);
                            }
                        });

                    }

                } catch (Exception jsonException) {
                    jsonException.printStackTrace();
                    if (reqCallback != null) {
                        okHandler.post(() -> reqCallback.onResponse(ErrorCodeConstants.GoChatErrorJSONFormatInvalid, "", null));
                    }
                }
            } else {
                if (reqCallback != null) {
                    okHandler.post(() -> reqCallback.onResponse(errorCode, "", null));
                }
            }
        });
    }
}