package com.zego.ktvchorus.rtc;

import com.zego.ktvchorus.rtc.callbacks.IZGKTVPlayerUpdateListener;
import com.zego.ktvchorus.rtc.callbacks.ZGKTVSongState;
import com.zego.ktvchorus.utils.L;

import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import im.zego.zegoexpress.ZegoCopyrightedMusic;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.ZegoMediaPlayer;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicDownloadCallback;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicEventHandler;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicGetStandardPitchCallback;
import im.zego.zegoexpress.callback.IZegoMediaPlayerEventHandler;
import im.zego.zegoexpress.callback.IZegoMediaPlayerLoadResourceCallback;
import im.zego.zegoexpress.callback.IZegoMediaPlayerSeekToCallback;
import im.zego.zegoexpress.constants.ZegoMediaPlayerNetworkEvent;
import im.zego.zegoexpress.constants.ZegoMediaPlayerState;

public class ZGKTVPlayerManager {

    public static final int ACCOMPANIMENT = 0;
    public static final int ORIGINAL_SONG = 1;
    //自动播放完成会设置次code
    public static final int AUTO_PLAY_END_CODE = -888;
    private static final String TAG = "ZGKTVPlayerManager";
    private ZegoCopyrightedMusic mCopyrightedMusic;
    private boolean enableAccompaniment = true;
    private int mVolumeNum = 50;

    private static volatile ZGKTVPlayerManager mInstance = new ZGKTVPlayerManager();

    private Timer mTimer;
    private TimerTask mTimerTask;

    private final Map<String, ZGKTVCopyrightedSong> mSongs = new HashMap<>();
    private final Map<String, String> mResourceIDSongs = new HashMap<>();

    private String mResourceID;
    private String mSongID;

    private ZegoMediaPlayer mediaPlayer;
    private IZGKTVPlayerUpdateListener mIZGKTVPlayerUpdateListener;

    private boolean isStop = false;
    private boolean isConformAccompany = false;

    private ZGKTVPlayerManager() {

    }

    public static ZGKTVPlayerManager getInstance() {
        if (mInstance == null) {
            synchronized (ZGKTVPlayerManager.class) {
                if (mInstance == null) {
                    mInstance = new ZGKTVPlayerManager();
                }
            }
        }
        return mInstance;
    }

    public void clearData() {
        L.i(TAG, "ZGKTVPlayerManager clearData");
        stop();
        setVolume(50);
        resetScore();
        mResourceID = null;
        mSongID = null;
        mSongs.clear();
        enableAccompaniment = true;
        mResourceIDSongs.clear();
        if (mTimerTask != null) {
            L.i(TAG, "stopTimerTask  1234");
            mTimerTask.cancel();
            mTimer.purge();
        }
    }


    public void init() {
        L.i(TAG, "ZGKTVPlayerManager init");

        if (mediaPlayer != null) {
            return;
        }

        mCopyrightedMusic = ZGKTVCopyrightedMusicManager.getInstance().getCopyrightedMusic();
        mediaPlayer = ZegoExpressEngine.getEngine().createMediaPlayer();
        mediaPlayer.setProgressInterval(60);
        mediaPlayer.setVolume(50);
        mediaPlayer.setEventHandler(new IZegoMediaPlayerEventHandler() {
            @Override
            public void onMediaPlayerStateUpdate(ZegoMediaPlayer mediaPlayer, ZegoMediaPlayerState state, int errorCode) {
                ZGKTVCopyrightedSong song = getSongByResourceID(mResourceID);
                L.i(TAG, "onPlaybackStateUpdate:state:" + state + ":errorCode:" + errorCode);
                if (state == ZegoMediaPlayerState.PLAYING) {
                    if (song != null) {
                        song.setState(ZGKTVSongState.PLAYING);
                    }
                    ZGKTVCopyrightedMusicManager.getInstance().resetScore(mResourceID);
                    ZGKTVCopyrightedMusicManager.getInstance().startScore(mResourceID);
                    ZGRealTimeKtvManager.getInstance().restartMvStream();
                } else if (state == ZegoMediaPlayerState.PAUSING) {
                    if (song != null) {
                        song.setState(ZGKTVSongState.PAUSING);
                    }
                    ZGKTVCopyrightedMusicManager.getInstance().pauseScore(mResourceID);
                } else if (state == ZegoMediaPlayerState.PLAY_ENDED) {
                    if (song != null) {
                        song.setState(ZGKTVSongState.PLAY_ENDED);
                    }
                    ZGKTVCopyrightedMusicManager.getInstance().stopScore(mResourceID);
                }
                if (song != null && song.getCallback() != null) {
                    song.getCallback().onPlaybackStateUpdate(state, errorCode);
                }

                if (mIZGKTVPlayerUpdateListener != null) {
                    mIZGKTVPlayerUpdateListener.onPlaybackStateUpdate(mediaPlayer, state, errorCode);
                }
            }

            @Override
            public void onMediaPlayerNetworkEvent(ZegoMediaPlayer
                                                          mediaPlayer, ZegoMediaPlayerNetworkEvent networkEvent) {
                super.onMediaPlayerNetworkEvent(mediaPlayer, networkEvent);
            }

            @Override
            public void onMediaPlayerPlayingProgress(ZegoMediaPlayer mediaPlayer, long millisecond) {
                L.i("PlayerManager", "onPlaybackProgressUpdate:progress:" + millisecond);
                ZGKTVCopyrightedSong song = getSongByResourceID(mResourceID);
                if (song != null && song.getCallback() != null) {
                    song.getCallback().onPlaybackProgressUpdate(millisecond);
                }

                if (mIZGKTVPlayerUpdateListener != null) {
                    mIZGKTVPlayerUpdateListener.onPlaybackProgressUpdate(mediaPlayer, millisecond);
                }
            }

            @Override
            public void onMediaPlayerRecvSEI(ZegoMediaPlayer mediaPlayer, byte[] data) {
                super.onMediaPlayerRecvSEI(mediaPlayer, data);
            }
        });
        mCopyrightedMusic.setEventHandler(new IZegoCopyrightedMusicEventHandler() {

            @Override
            public void onCurrentPitchValueUpdate(ZegoCopyrightedMusic copyrightedMusic, String
                    resourceID, int currentDuration, int pitchValue) {
                super.onCurrentPitchValueUpdate(copyrightedMusic, resourceID, currentDuration, pitchValue);
                if (mIZGKTVPlayerUpdateListener != null) {
                    mIZGKTVPlayerUpdateListener.onCurrentPitchValueUpdate(resourceID, currentDuration, pitchValue);
                }
            }

            @Override
            public void onDownloadProgressUpdate(ZegoCopyrightedMusic copyrightedMusic, String
                    resourceID, float progressRate) {
                super.onDownloadProgressUpdate(copyrightedMusic, resourceID, progressRate);
                L.i(TAG, "onLoadProgressUpdate:resourceID:" + resourceID + ":progressRate:" + progressRate);
                ZGKTVCopyrightedSong song = getSongByResourceID(resourceID);
                if (song != null && song.getCallback() != null) {
                    if (song.getState() == ZGKTVSongState.NO_LOAD && progressRate != 0) {
                        song.setState(ZGKTVSongState.LOADING);
                        if (progressRate == 1) {
                            song.setState(ZGKTVSongState.NO_PLAY);
                        }
                    }
                    song.getCallback().onLoadProgressUpdate(progressRate);
                }

                if (song != null) {
                    ZGKTVSongLoadManager.getInstance().loading(song.getSongID(), progressRate);
                }
            }
        });
        mTimer = new Timer();
    }


    public void uninit() {
        mIZGKTVPlayerUpdateListener = null;
        if (mTimerTask != null) {
            mTimerTask.cancel();
        }
        if (mTimer != null) {
            mTimer.purge();
            mTimer.cancel();
        }
        if (mediaPlayer != null) {
            mediaPlayer.stop();
            mediaPlayer.setEventHandler(null);
            ZegoExpressEngine.getEngine().destroyMediaPlayer(mediaPlayer);
            mediaPlayer = null;
        }
    }


    public void putSong(String songID, ZGKTVCopyrightedSong song) {
        L.i("songId", "putSong : " + songID + " : : " + song.getLoadState());
        mSongs.put(songID, song);
    }

    public ZGKTVCopyrightedSong getSong(String songID) {
        return mSongs.get(songID);
    }

    public ZGKTVCopyrightedSong getSongByResourceID(String resourceID) {
        return mSongs.get(mResourceIDSongs.get(resourceID));
    }

    public void putResourceIDSong(String resourceID, String songID) {
        mResourceIDSongs.put(resourceID, songID);
    }

    public void load(String songID, IZegoCopyrightedMusicDownloadCallback callback) {
        ZGKTVCopyrightedSong song = mSongs.get(songID);
        if (song == null) {
            L.e(TAG, "Load Error: songID " + songID + " is NULL");
            return;
        }

        mResourceID = song.getResourceID();
        L.i("songId", " load :: songID " + songID + " resourceID " + mResourceID);
        if (mResourceID != null) {
            ZGKTVSongLoadManager.getInstance().startLoad(songID);
            mCopyrightedMusic.download(mResourceID, new IZegoCopyrightedMusicDownloadCallback() {
                @Override
                public void onDownloadCallback(int errorCode) {
                    L.i("songId", " load :: songID " + songID + " resourceID " + mResourceID + "errorCode :" + errorCode);
                    if (errorCode != 0) {
                        ZGKTVSongLoadManager.getInstance().loadError(songID, errorCode);
                    }
                    if (callback != null) {
                        callback.onDownloadCallback(errorCode);
                    }
                }
            });
        }
        if (ZGKTVLyricManager.getInstance().getLyric(songID) == null) {
            ZGKTVCopyrightedMusicManager.getInstance().getKrcLyricForSongID(songID, null);
        }
    }

    public void play(String songID) {
        play(songID, 0);
    }

    public void play(String songID, long position) {
        String resourceID = getSong(songID).getResourceID();
        L.i(TAG, "play: startPlay :songID:" + songID + ":position:" + position);
        if (mResourceID != null) {
            L.e(TAG, "play: stop :mResourceID:" + mResourceID + ":resourceID:" + resourceID);
            L.i(TAG, "stopTimerTask 9999");
            stopTimerTask();
            stop();
        }
        L.i(TAG, "play: playing :songID:" + songID + ":position:" + position);
        mSongID = songID;
        loadResourceAndStart(resourceID, position);
    }

    private void loadResourceAndStart(String resourceID, long position) {
        if (resourceID != null) {
            mResourceID = resourceID;
            isStop = false;
            //            mCopyrightedMusic.play(resourceID,position,mediaPlayer);
            mediaPlayer.loadCopyrightedMusicResourceWithPosition(resourceID, position, new IZegoMediaPlayerLoadResourceCallback() {

                @Override
                public void onLoadResourceCallback(int errorCode) {
                    if (errorCode == 0) {
                        if (mResourceID == null) {
                            L.e(TAG, "onLoadResourceCallback mResourceID is ERROR :" + mResourceID);
                        }
                        mediaPlayer.start();
                        setAudioOrigin(enableAccompaniment);
                    } else {
                        L.e(TAG, "loadCopyrightedMusicResourceWithPosition Error! ErrorCOde = " + errorCode);
                    }
                }
            });
        }
    }

    public void seekTo(long position) {
        L.i(TAG, "seekTo: seekTo :mResourceID:" + mResourceID + ":position:" + position);
        if (mResourceID != null) {
            mediaPlayer.seekTo(position, new IZegoMediaPlayerSeekToCallback() {
                @Override
                public void onSeekToTimeCallback(int errorCode) {

                }
            });
        }
    }

    public void pause() {
        L.i(TAG, "pause: pause :mResourceID:" + mResourceID);
        if (mResourceID != null) {
            mediaPlayer.pause();
            ZGKTVCopyrightedMusicManager.getInstance().pauseScore(mResourceID);
        }
    }

    public void resume() {
        L.i(TAG, "resume: resume :mResourceID:" + mResourceID);
        if (mResourceID != null) {
            mediaPlayer.resume();
            ZGKTVCopyrightedMusicManager.getInstance().resumeScore(mResourceID);
        }
    }

    public void stop() {
        L.i(TAG, "stop: stop :mResourceID:" + mResourceID);
        if (mResourceID != null) {
            ZGKTVCopyrightedSong song = getSongByResourceID(mResourceID);
            if (song != null) {
                song.setState(ZGKTVSongState.NO_PLAY);
            }
            isStop = true;
            ZGKTVCopyrightedMusicManager.getInstance().stopScore(mResourceID);
            ZGKTVCopyrightedMusicManager.getInstance().resetScore(mResourceID);
            if (mediaPlayer != null) {
                mediaPlayer.stop();
            }
            mResourceID = null;
            isConformAccompany = false;
        }
    }

    public void stopTimerTask() {
        L.i(TAG, "stopTimerTask");
        if (mTimerTask != null) {
            mTimerTask.cancel();
            mTimer.purge();
        }
    }

    public boolean isPlaying() {
        ZGKTVCopyrightedSong song = getSongByResourceID(mResourceID);
        if (song == null) {
            return false;
        }
        return song.getState() == ZGKTVSongState.PLAYING;
    }

    public boolean isNoPlay() {
        ZGKTVCopyrightedSong song = getSongByResourceID(mResourceID);
        if (song == null) {
            return false;
        }
        return song.getState() == ZGKTVSongState.NO_PLAY || song.getState() == ZGKTVSongState.NO_LOAD;
    }

    /**
     * 设置原唱伴唱的音轨
     *
     * @param enableAccompaniment
     */
    public void setAudioOrigin(boolean enableAccompaniment) {
        this.enableAccompaniment = enableAccompaniment;
        if (mResourceID != null) {
            L.i(TAG, "setAudioOrigin: setAudioOrigin :mResourceID:" + mResourceID + ":enableAccompaniment:" + enableAccompaniment);
            //            mCopyrightedMusic.setAudioTrackIndex(mediaPlayer, enableAccompaniment ? ACCOMPANIMENT : ORIGINAL_SONG);
            mediaPlayer.setAudioTrackIndex(enableAccompaniment ? ACCOMPANIMENT : ORIGINAL_SONG);
        }
    }

    public void startInFuture(final long ntpTimestamp, String resourceID) {
        if (mResourceID != null) {
            L.e(TAG, "play: stop :mResourceID:" + mResourceID + ":resourceID:" + resourceID);
            stopTimerTask();
            stop();
        }
        mResourceID = resourceID;
        isStop = false;
        mediaPlayer.loadCopyrightedMusicResourceWithPosition(resourceID, 0, errorCode -> {
            if (errorCode == 0) {
                setAudioOrigin(enableAccompaniment);
            } else {
                L.e(TAG, "loadCopyrightedMusicResourceWithPosition Error! ErrorCode = " + errorCode);
            }
        });
        TimerTask timerTask = new TimerTask() {
            @Override
            public void run() {
                mediaPlayer.start();
            }
        };
        delayPerformWithTimestamp(ntpTimestamp, timerTask);
    }

    public void conformAccompany(String resourceID, long progress, long ntpTimestamp) {
        if (isConformAccompany) {
            return;
        }
        isConformAccompany = true;
        mResourceID = resourceID;
        isStop = false;
        mediaPlayer.loadCopyrightedMusicResourceWithPosition(resourceID, 0, errorCode -> {
            isConformAccompany = false;
            if (errorCode == 0) {
                mediaPlayer.start();
                setAudioOrigin(enableAccompaniment);
                long curNtp = ZGRealTimeKtvManager.getInstance().getNetworkTime();
                long interval = ntpTimestamp == 0 ? progress : (curNtp - ntpTimestamp + progress);
                seekTo(interval);
            } else {
                L.e(TAG, "loadCopyrightedMusicResourceWithPosition Error! ErrorCode = " + errorCode);
            }
        });
    }

    public long getCurrentProgress() {
        return mediaPlayer.getCurrentProgress();
    }

    /**
     * 设置于未来精确时间启动的timer任务
     *
     * @param startTime 启始时间
     * @param timerTask timer任务
     */
    private void delayPerformWithTimestamp(long startTime, TimerTask timerTask) {
        if (mTimerTask != null) {
            L.i(TAG, "stopTimerTask 0000");
            mTimerTask.cancel();
            mTimer.purge();
        }
        mTimerTask = timerTask;
        long currentTime = ZGRealTimeKtvManager.getInstance().getNetworkTime();
        long executeDelay = startTime - currentTime;
        if (executeDelay < 0) {
            L.e(TAG, "start time " + startTime + "is smaller than current time" + currentTime);
            executeDelay = 0;
        } else {
            L.i(TAG, "currentTime " + currentTime + " executeDelay " + executeDelay);
        }
        mTimer.schedule(mTimerTask, executeDelay);
    }

    public void setOnIZGKTVPlayerUpdateListener(IZGKTVPlayerUpdateListener iZGKTVPlayerUpdateListener) {
        this.mIZGKTVPlayerUpdateListener = iZGKTVPlayerUpdateListener;
    }

    /**
     * 设置媒体播放器播放音量
     *
     * @param volume 播放音量 0-200
     */
    public void setVolume(int volume) {
        L.i(TAG, "setVolume :volume:" + volume);
        this.mVolumeNum = volume;
        if (mediaPlayer != null) {
            mediaPlayer.setVolume(volume);
        }
    }

    public int getVolumeNum() {
        return mVolumeNum;
    }

    /**
     * 获取标准音高线
     *
     * @return
     */
    public void getStandardPitch(IZegoCopyrightedMusicGetStandardPitchCallback callback) {
        if (mResourceID != null) {
            ZGKTVCopyrightedMusicManager.getInstance().getStandardPitch(mResourceID, callback);
        } else {

        }
    }


    /**
     * 获取前一句分数
     *
     * @return
     */
    public int getPreviousScore() {
        if (mResourceID != null) {
            return ZGKTVCopyrightedMusicManager.getInstance().getPreviousScore(mResourceID);
        } else {
            return 0;
        }
    }

    /**
     * 获取当前音高值
     *
     * @return
     */
    public int getCurrentPitch() {
        if (mResourceID != null) {
            return ZGKTVCopyrightedMusicManager.getInstance().getCurrentPitch(mResourceID);
        } else {
            return 0;
        }
    }

    /**
     * 获取平均分数
     *
     * @return
     */
    public int getAverageScore() {
        if (mResourceID != null) {
            return ZGKTVCopyrightedMusicManager.getInstance().getAverageScore(mResourceID);
        } else {
            return -1;
        }
    }

    /**
     * 获取总分数
     *
     * @return
     */
    public int getTotalScore() {
        if (mResourceID != null) {
            return ZGKTVCopyrightedMusicManager.getInstance().getTotalScore(mResourceID);
        } else {
            return -1;
        }
    }

    public void resetScore() {
        if (mResourceID != null) {
            ZGKTVCopyrightedMusicManager.getInstance().resetScore(mResourceID);
        }
    }

    public void pauseScore() {
        L.i(TAG, "pauseScore: pauseScore :mResourceID:" + mResourceID);
        if (mResourceID != null) {
            ZGKTVCopyrightedMusicManager.getInstance().pauseScore(mResourceID);
        }
    }

    public void resumeScore() {
        L.i(TAG, "resumeScore: resumeScore :mResourceID:" + mResourceID);
        if (mResourceID != null) {
            ZGKTVCopyrightedMusicManager.getInstance().resumeScore(mResourceID);
        }
    }

    public long getTotalDuration() {
        if (mediaPlayer != null) {
            return mediaPlayer.getTotalDuration();
        }
        return 0;
    }

    public String getCurrentPlaySongID() {
        return mSongID;
    }
}
