package com.zego.ktvchorus.rtc;

import static im.zego.zegoexpress.constants.ZegoRoomState.CONNECTED;
import static im.zego.zegoexpress.constants.ZegoUpdateType.ADD;
import static im.zego.zegoexpress.constants.ZegoUpdateType.DELETE;

import android.app.Application;
import android.content.Context;
import android.graphics.Rect;
import android.util.Log;

import com.google.gson.Gson;
import com.zego.ktvchorus.model.Constants;
import com.zego.ktvchorus.model.ErrorCodeConstants;
import com.zego.ktvchorus.model.HostSEIInfo;
import com.zego.ktvchorus.model.NetworkQuality;
import com.zego.ktvchorus.model.RoomMusicInfo;
import com.zego.ktvchorus.model.StreamExtraInfo;
import com.zego.ktvchorus.model.UserDataMonitorInfo;
import com.zego.ktvchorus.model.UserInfo;
import com.zego.ktvchorus.model.ZGRoomIdentifier;
import com.zego.ktvchorus.model.ZegoDataCenter;
import com.zego.ktvchorus.rtc.callbacks.IEventCallback;
import com.zego.ktvchorus.rtc.callbacks.IRTCCommonCallback;
import com.zego.ktvchorus.utils.GlideImageLoader;
import com.zego.ktvchorus.utils.L;
import com.zego.ktvchorus.utils.T;

import org.jetbrains.annotations.NotNull;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoEventHandler;
import im.zego.zegoexpress.callback.IZegoMixerStartCallback;
import im.zego.zegoexpress.constants.ZegoAECMode;
import im.zego.zegoexpress.constants.ZegoANSMode;
import im.zego.zegoexpress.constants.ZegoAudioChannel;
import im.zego.zegoexpress.constants.ZegoAudioCodecID;
import im.zego.zegoexpress.constants.ZegoAudioConfigPreset;
import im.zego.zegoexpress.constants.ZegoAudioSourceType;
import im.zego.zegoexpress.constants.ZegoLanguage;
import im.zego.zegoexpress.constants.ZegoMixerInputContentType;
import im.zego.zegoexpress.constants.ZegoNetworkMode;
import im.zego.zegoexpress.constants.ZegoNetworkSpeedTestType;
import im.zego.zegoexpress.constants.ZegoPublishChannel;
import im.zego.zegoexpress.constants.ZegoRemoteDeviceState;
import im.zego.zegoexpress.constants.ZegoReverbPreset;
import im.zego.zegoexpress.constants.ZegoRoomState;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zegoexpress.constants.ZegoStreamAlignmentMode;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoAudioConfig;
import im.zego.zegoexpress.entity.ZegoCustomAudioConfig;
import im.zego.zegoexpress.entity.ZegoEngineConfig;
import im.zego.zegoexpress.entity.ZegoEngineProfile;
import im.zego.zegoexpress.entity.ZegoMixerAudioConfig;
import im.zego.zegoexpress.entity.ZegoMixerInput;
import im.zego.zegoexpress.entity.ZegoMixerOutput;
import im.zego.zegoexpress.entity.ZegoMixerTask;
import im.zego.zegoexpress.entity.ZegoNetworkTimeInfo;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublisherConfig;
import im.zego.zegoexpress.entity.ZegoRoomExtraInfo;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zegoexpress.entity.ZegoUser;

public class ZGRealTimeKtvManager {
    private static final String TAG = "ZGRealTimeKtvManager";

    private int mHeadphoneMonitorVolumeNum = 50;
    private int mCaptureVolumeNum = 100;
    private boolean mLoopbackSwitch = false;
    private int mVerbPresetPos = 3;
    private ZGRoomIdentifier mRole;

    private ZGRealTimeKtvManager() {
    }

    public void initAudioProcessParams() {
        setCaptureVolume(100);  //设置推流端采集音量，范围为 0 ~ 200，默认值为 100
        setLoopBackVolume(50);
        enableHeadphoneMonitor(false); //耳返开关默认关闭
        mSDKEngine.enableAEC(true);  //是否开启回声消除；true 表示开启；false 表示关闭
        mSDKEngine.enableAGC(false);  //是否开启自动增益控制；true 表示开启；false 表示关闭
        mSDKEngine.enableANS(true);  //是否开启噪声抑制；true 表示开启噪声抑制；false 表示关闭噪声抑制
        mSDKEngine.setAECMode(ZegoAECMode.MEDIUM); //设置回声消除模式
        mSDKEngine.setANSMode(ZegoANSMode.MEDIUM); //设置音频噪声抑制模式
        mSDKEngine.enableHeadphoneMonitor(false);  //开启耳返， false: 关闭耳返
        mSDKEngine.setReverbPreset(ZegoReverbPreset.RECORDING_STUDIO); //通过预设枚举设置混响  RECORDING_STUDIO - 录音棚混响效果
    }

    //开启耳返， false: 关闭耳返
    public void enableHeadphoneMonitor(boolean enable) {
        L.i(TAG, "enableHeadphoneMonitor :" + enable);
        this.mLoopbackSwitch = enable;
        mSDKEngine.enableHeadphoneMonitor(enable);
    }

    //设置混响
    public void setVerbPresetMode(ZegoReverbPreset mode, int position) {
        L.i(TAG, "setVerbPresetMode :" + mode);
        this.mVerbPresetPos = position;
        mSDKEngine.setReverbPreset(mode);
    }

    private static final class Holder {
        private static final ZGRealTimeKtvManager INSTANCE = new ZGRealTimeKtvManager();
    }

    public static ZGRealTimeKtvManager getInstance() {
        return Holder.INSTANCE;
    }

    private ZegoExpressEngine mSDKEngine = null;
    private IEventCallback mEventCallbackList;
    private ZegoUser mUserInfo = null;
    private String mRoomID = null;
    public static final int MAX_USER_COUNT = 10000000;

    //台上用户和对应的流
    private final HashMap<String, String> mUserStreamIDMap = new HashMap<>();
    private final HashMap<String, String> mUserNameIDMap = new HashMap<>();
    //声浪集合
    private final HashMap<String, Float> mUserVolumeMap = new HashMap<>();
    private ArrayList<UserInfo> mUserInfoList = new ArrayList<>();

    public void addListener(IEventCallback callback) {
        mEventCallbackList = callback;
        if (isAnchor()) {
            UserInfo userInfo = new UserInfo();
            userInfo.setUid(ZegoDataCenter.USER_ID);
            userInfo.setRoomID(mRoomID);
            userInfo.setNickName(ZegoDataCenter.getUserName());
            userInfo.setMicIndex(0);
            userInfo.setAvatar(GlideImageLoader.getInstance()
                    .getDrawableIdByTimestamp(ZegoDataCenter.USER_ID));
            mUserInfoList.add(userInfo);
        }
        mEventCallbackList.onUserListUpdate(mUserInfoList);
    }

    // 初始化引擎
    public void init(@NotNull Context application, IRTCCommonCallback callback) {
        if (mSDKEngine != null) {
            callback.onRTCCallback(0);
            return;
        }

        preEngineConfig();

        ZegoEngineProfile profile = new ZegoEngineProfile();
        profile.appID = ZegoDataCenter.APP_ID;
        profile.appSign = ZegoDataCenter.APP_SIGN;
        profile.scenario = ZegoScenario.GENERAL;
        profile.application = (Application) application.getApplicationContext();

        mSDKEngine = ZegoExpressEngine.createEngine(profile, createEventHandler());
        Log.d("pengliang", "version " + ZegoExpressEngine.getVersion());

        if (mSDKEngine != null) {
            engineConfig();
            ZGKTVCopyrightedMusicManager.getInstance().setUserID(mUserInfo.userID, mUserInfo.userName);
            ZGKTVCopyrightedMusicManager.getInstance().init(application, callback);
        } else {
            callback.onRTCCallback(0);
            T.show("引擎创建失败，请检查appID ，appSign是否存在问题");
        }
    }

    private IZegoEventHandler createEventHandler() {
        return new IZegoEventHandler() {

            // 避免首次进入弹出分数弹窗
            private boolean stopFlag = true;

            @Override
            public void onDebugError(int errorCode, String funcName, String info) {
                super.onDebugError(errorCode, funcName, info);

                if (errorCode == 1017041) {
                    T.show(info);
                }

                L.e(TAG, "onDebugError-:errorCode:" + errorCode + ":funcName:" + funcName);
                L.e(TAG, "onDebugError-:errorCode:" + errorCode + ":info:" + info);
            }

            @Override
            public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData) {
                for (ZegoStream stream : streamList) {
                    L.e(TAG, "onRoomStreamUpdate:" + " : updateType : " + updateType + " : streamID : " + stream.streamID);
                    if (updateType == ADD) {
                        if (!stream.streamID.contains("_mv") && !getMainStreamId().contains(stream.streamID)) {
                            if (isAnchor()) {
                                mSDKEngine.setPlayStreamBufferIntervalRange(stream.streamID, 30, 30);
                                mSDKEngine.startPlayingStream(stream.streamID);
                            }
                            mUserStreamIDMap.put(stream.user.userID, stream.streamID);
                            mUserNameIDMap.put(stream.user.userID, stream.user.userName);
                        }
                    } else if (updateType == DELETE) {
                        if (!stream.streamID.contains("_mv")) {
                            mSDKEngine.stopPlayingStream(stream.streamID);
                            mUserStreamIDMap.remove(stream.user.userID);
                            mUserNameIDMap.remove(stream.user.userID);
                        }
                    }
                }
                if (mEventCallbackList != null) {
                    // zego stream to UserInfo
                    mUserInfoList = new ArrayList<>();
                    int position = 0;
                    if (isAnchor()) {
                        UserInfo userInfo = new UserInfo();
                        userInfo.setUid(ZegoDataCenter.USER_ID);
                        userInfo.setRoomID(roomID);
                        userInfo.setNickName(ZegoDataCenter.getUserName());
                        userInfo.setMicIndex(position);
                        userInfo.setAvatar(GlideImageLoader.getInstance()
                                .getDrawableIdByTimestamp(ZegoDataCenter.USER_ID));
                        position++;
                        mUserInfoList.add(userInfo);
                    }
                    for (String userId : mUserStreamIDMap.keySet()) {
                        UserInfo userInfo = new UserInfo();
                        userInfo.setUid(Long.parseLong(userId));
                        userInfo.setRoomID(roomID);
                        userInfo.setNickName(mUserNameIDMap.get(userId));
                        userInfo.setMicIndex(position);
                        userInfo.setAvatar(GlideImageLoader.getInstance()
                                .getDrawableIdByTimestamp(Long.parseLong(userId)));
                        position++;
                        mUserInfoList.add(userInfo);
                    }
                    mEventCallbackList.onUserListUpdate(mUserInfoList);
                }
                if (mRole == ZGRoomIdentifier.ZGRoomAnchor) {
                    updateMixStream();
                }
            }

            /**
             * 本地采集音频声浪回调
             *
             * 回调通知周期为 100 ms。
             * @param soundLevel 本地采集的声浪值，取值范围为 0.0 ~ 100.0
             */
            @Override
            public void onCapturedSoundLevelUpdate(float soundLevel) {
                if (mSDKEngine.isMicrophoneMuted()) {
                    return;
                }
                mUserVolumeMap.clear();
                if (mUserInfo.userID == null) {
                    return;
                }
                int length = mUserInfo.userID.length();
                String key = String.valueOf(mUserInfo.userID);
                if (length >= Constants.SOUND_ID_SURFIX_NUM) {
                    key = (key.substring(length - Constants.SOUND_ID_SURFIX_NUM, length));
                }
                mUserVolumeMap.put(key, soundLevel);
            }

            @Override
            public void onMixerSoundLevelUpdate(HashMap<Integer, Float> soundLevels) {
                mUserVolumeMap.clear();
                // 根据流id 更新对应的user的音量
                for (Map.Entry<Integer, Float> pair : soundLevels.entrySet()) {
                    mUserVolumeMap.put(pair.getKey().toString(), pair.getValue());
                }
            }

            @Override
            public void onPlayerRecvSEI(String streamID, byte[] data) {
                // 伴奏对齐
                HostSEIInfo hostSEIInfo = APIBase.getGson().fromJson(new String(data), HostSEIInfo.class);
                Log.d("pengliang", "hostSEIInfo -> " + hostSEIInfo);
                if (isAnchor()) {
                    calculateAccompanyLatencyWithDict(hostSEIInfo);
                    syncAccompanyWithDict(hostSEIInfo);
                }
                if (mEventCallbackList != null) {
                    mEventCallbackList.onPlayerRecvSEI(streamID, hostSEIInfo);
                }
            }

            @Override
            public void onPublisherQualityUpdate(String streamId, ZegoPublishStreamQuality zegoPublishStreamQuality) {
                UserDataMonitorInfo userDataMonitorInfo = new UserDataMonitorInfo();
                NetworkQuality rtt, plr;
                if (zegoPublishStreamQuality.rtt < 40) {
                    rtt = NetworkQuality.GRATE;
                    userDataMonitorInfo.setNetworkQuality(NetworkQuality.GRATE);
                } else if (zegoPublishStreamQuality.rtt < 60) {
                    rtt = NetworkQuality.FINE;
                } else {
                    rtt = NetworkQuality.BAD;
                }
                if (zegoPublishStreamQuality.packetLostRate < 10) {
                    plr = NetworkQuality.GRATE;
                } else if (zegoPublishStreamQuality.packetLostRate < 15) {
                    plr = NetworkQuality.FINE;
                } else {
                    plr = NetworkQuality.BAD;
                }
                if (rtt == NetworkQuality.GRATE && plr == NetworkQuality.GRATE) {
                    userDataMonitorInfo.setNetworkQuality(NetworkQuality.GRATE);
                } else if (rtt == NetworkQuality.BAD && plr == NetworkQuality.BAD) {
                    userDataMonitorInfo.setNetworkQuality(NetworkQuality.BAD);
                } else {
                    userDataMonitorInfo.setNetworkQuality(NetworkQuality.FINE);
                }
                if (mEventCallbackList != null) {
                    mEventCallbackList.onPlayQuality(streamId, userDataMonitorInfo);
                }
            }

            @Override
            public void onPlayerQualityUpdate(String streamId, ZegoPlayStreamQuality zegoPlayStreamQuality) {
                UserDataMonitorInfo userDataMonitorInfo = new UserDataMonitorInfo();
                NetworkQuality rtt, plr;
                if (zegoPlayStreamQuality.rtt < 40) {
                    rtt = NetworkQuality.GRATE;
                    userDataMonitorInfo.setNetworkQuality(NetworkQuality.GRATE);
                } else if (zegoPlayStreamQuality.rtt < 60) {
                    rtt = NetworkQuality.FINE;
                } else {
                    rtt = NetworkQuality.BAD;
                }
                if (zegoPlayStreamQuality.packetLostRate < 10) {
                    plr = NetworkQuality.GRATE;
                } else if (zegoPlayStreamQuality.packetLostRate < 15) {
                    plr = NetworkQuality.FINE;
                } else {
                    plr = NetworkQuality.BAD;
                }
                if (rtt == NetworkQuality.GRATE && plr == NetworkQuality.GRATE) {
                    userDataMonitorInfo.setNetworkQuality(NetworkQuality.GRATE);
                } else if (rtt == NetworkQuality.BAD && plr == NetworkQuality.BAD) {
                    userDataMonitorInfo.setNetworkQuality(NetworkQuality.BAD);
                } else {
                    userDataMonitorInfo.setNetworkQuality(NetworkQuality.FINE);
                }
                if (mEventCallbackList != null) {
                    mEventCallbackList.onPlayQuality(streamId, userDataMonitorInfo);
                }
            }

            @Override
            public void onRemoteMicStateUpdate(String userId, ZegoRemoteDeviceState zegoRemoteDeviceState) {
                if (mEventCallbackList != null) {
                    for (UserInfo userInfo : mUserInfoList) {
                        if (userId.equals(String.valueOf(userInfo.getUserID()))) {
                            userInfo.setMicOpen(zegoRemoteDeviceState.value() == 0);
                            mEventCallbackList.onUserListUpdate(mUserInfoList);
                            break;
                        }
                    }
                }
            }

            @Override
            public void onRoomStreamExtraInfoUpdate(String roomId, ArrayList<ZegoStream> streamList) {
                StreamExtraInfo streamExtraInfo = APIBase.getGson().fromJson(
                        streamList.get(0).extraInfo, StreamExtraInfo.class);
                if (streamExtraInfo != null && !streamExtraInfo.getMessage().equals("LeaveRoom")) {
                    if (mRole == ZGRoomIdentifier.ZGRoomAudience) {
                        return;
                    }
                    if (streamExtraInfo.getkBehaviorType() == Constants.MediaPlayerBehavior.PLAYING.value()) {
                        ZGKTVPlayerManager.getInstance().startInFuture(streamExtraInfo.getkStartTime(),
                                streamExtraInfo.getkResourceID());
                        if (mEventCallbackList != null) {
                            mEventCallbackList.onMusicStart();
                        }
                    } else if (streamExtraInfo.getkBehaviorType() == Constants.MediaPlayerBehavior.STOP.value()) {
                        if (!stopFlag && mEventCallbackList != null) {
                            mEventCallbackList.onMusicStop(streamExtraInfo.getStopType());
                        }
                        stopFlag = false;
                        ZGKTVPlayerManager.getInstance().stop();
                    }
                }
            }

            private void syncAccompanyWithDict(HostSEIInfo hostSEIInfo) {
                if (ZGKTVPlayerManager.getInstance().isNoPlay()) {
                    ZGKTVPlayerManager.getInstance().conformAccompany(
                            hostSEIInfo.getkResourceID(),
                            hostSEIInfo.getkProgressKey(),
                            hostSEIInfo.getPointTimeKey()
                    );
                }
            }

            private void calculateAccompanyLatencyWithDict(HostSEIInfo hostSEIInfo) {
                long oppositeProgress = hostSEIInfo.getProcessMillis();
                long oppositeTime = hostSEIInfo.getPointTimeKey();
                long ntp = mSDKEngine.getNetworkTimeInfo().timestamp;
                long currentProgress = ZGKTVPlayerManager.getInstance().getCurrentProgress();
                long latency = ntp - oppositeTime - (currentProgress - oppositeProgress);
                if (latency > 30) {
                    ZGKTVPlayerManager.getInstance().seekTo(currentProgress + latency);
                }
            }

            /**
             * 远端拉流音频声浪回调
             *
             * 回调通知周期为 100 ms。
             * @param soundLevels 远端的声浪键值对，key 为流 ID，value 为对应的流的声浪值，value 取值范围为 0.0 ~ 100.0
             */
            @Override
            public void onRemoteSoundLevelUpdate(HashMap<String, Float> soundLevels) {
                mUserVolumeMap.clear();
                // 根据流id 更新对应的user的音量
                for (Map.Entry<String, Float> pair : soundLevels.entrySet()) {
                    if (!pair.getKey().endsWith(Constants.MIX_STREAM_SURFIX)) {
                        int length = pair.getKey().length();
                        String key = pair.getKey();
                        if (length >= Constants.SOUND_ID_SURFIX_NUM) {
                            key = (key.substring(length - Constants.SOUND_ID_SURFIX_NUM, length));
                        }
                        mUserVolumeMap.put(key, pair.getValue());
                    }
                }
            }

            @Override
            public void onNetworkSpeedTestError(int errorCode, ZegoNetworkSpeedTestType type) {

            }

            @Override
            public void onNetworkModeChanged(ZegoNetworkMode mode) {
                super.onNetworkModeChanged(mode);
            }

            @Override
            public void onRoomStateUpdate(String roomID, ZegoRoomState state, int errorCode, JSONObject extendedData) {
                if (state == CONNECTED && errorCode == 0) {
                    // 观众进房默认拉混流 上台后停止拉混流
                    if (isAnchor()) {
                        startPublishMV();
                        startPublishAudio();
                    } else {
                        startPlayingAux();
                    }
                }
            }

            @Override
            public void onRoomExtraInfoUpdate(String s, ArrayList<ZegoRoomExtraInfo> arrayList) {
                for (ZegoRoomExtraInfo zegoRoomExtraInfo : arrayList) {
                    if (zegoRoomExtraInfo.key.equals("RoomMusicInfo")) {
                        RoomMusicInfo roomMusicInfo = new Gson().fromJson(zegoRoomExtraInfo.value, RoomMusicInfo.class);
                        FileDownloadManager.getInstance().downloadFile(roomMusicInfo.getSongID(), roomMusicInfo, true);
                    }
                }
            }
        };
    }

    //引擎配置
    private void engineConfig() {
        // 不需要视频功能 关闭视频功能
        mSDKEngine.startSoundLevelMonitor(500);
        mSDKEngine.enableCamera(false);
        mSDKEngine.enableCamera(false, ZegoPublishChannel.AUX);

        // 第二路流配置媒体播放器
        ZegoCustomAudioConfig zegoCustomAudioConfig = new ZegoCustomAudioConfig();
        zegoCustomAudioConfig.sourceType = ZegoAudioSourceType.MEDIA_PLAYER;
        mSDKEngine.enableCustomAudioIO(true, zegoCustomAudioConfig, ZegoPublishChannel.AUX);
        mSDKEngine.setDebugVerbose(true, ZegoLanguage.CHINESE);

        ZegoAudioConfig audioConfig = new ZegoAudioConfig(ZegoAudioConfigPreset.HIGH_QUALITY);
        audioConfig.bitrate = 64;
        audioConfig.codecID = ZegoAudioCodecID.LOW3;
        audioConfig.channel = ZegoAudioChannel.MONO;
        mSDKEngine.setStreamAlignmentProperty(1, ZegoPublishChannel.MAIN);
        mSDKEngine.setAudioConfig(audioConfig, ZegoPublishChannel.MAIN);

        ZegoAudioConfig auxAudioConfig = new ZegoAudioConfig(ZegoAudioConfigPreset.HIGH_QUALITY);
        audioConfig.bitrate = 128;
        audioConfig.codecID = ZegoAudioCodecID.LOW3;
        audioConfig.channel = ZegoAudioChannel.MONO;
        mSDKEngine.setStreamAlignmentProperty(1, ZegoPublishChannel.AUX);
        mSDKEngine.setAudioConfig(auxAudioConfig, ZegoPublishChannel.AUX);
    }

    //初始化引擎前配置
    private void preEngineConfig() {
        // 配置日志路径
        ZegoEngineConfig engineConfig = new ZegoEngineConfig();
        // 开启超低延迟模式
        engineConfig.advancedConfig.put("ultra_low_latency", "true");
        // 开启超低采集和渲染耗时模式
        engineConfig.advancedConfig.put("enforce_audio_loopback_in_sync", "true");
        ZegoExpressEngine.setEngineConfig(engineConfig);
    }

    // 反初始化
    public void uninit() {
        if (mSDKEngine == null) {
            return;
        }
        mUserInfoList.clear();
        mUserStreamIDMap.clear();
        mUserNameIDMap.clear();
        mEventCallbackList = null;
        mSDKEngine.setEventHandler(null);
        ZegoExpressEngine.destroyEngine(null);
        mSDKEngine = null;
    }

    //设置用户信息，需在登录前设置
    public void setUserID(long userID, @NotNull String userName) {
        mUserInfo = new ZegoUser(String.valueOf(userID), userName);
    }

    // 登录房间
    public void joinRoomWithRoomID(@NotNull String roomID, ZGRoomIdentifier role) {
        mSDKEngine.muteMicrophone(true);
        initAudioProcessParams();

        mUserStreamIDMap.clear();
        mUserNameIDMap.clear();
        this.mRoomID = roomID;
        this.mRole = role;

        mSDKEngine.loginRoom(roomID, mUserInfo);
    }

    //退出房间
    public void logoutRoom() {
        if (mSDKEngine == null) {
            return;
        }

        if (mRoomID != null) {
            mSDKEngine.logoutRoom(mRoomID);
            //            mSDKEngine.setEventHandler(null);
            mRoomID = null;
            mUserStreamIDMap.clear();
            mUserNameIDMap.clear();
        }
        if (mSDKEngine != null) {
            mSDKEngine.stopSoundLevelMonitor();
        }
    }

    //开始推音频流
    public void startPublishAudio() {
        L.i(TAG, "startPublishAudio");
        ZegoPublisherConfig config = new ZegoPublisherConfig();
        config.forceSynchronousNetworkTime = 1;
        config.roomID = mRoomID;
        mSDKEngine.startPublishingStream(getMainStreamId(), config, ZegoPublishChannel.MAIN);
    }

    //开始推伴奏流
    public void startPublishMV() {
        L.i(TAG, "startPublishAudio AUX");
        ZegoPublisherConfig config = new ZegoPublisherConfig();
        config.forceSynchronousNetworkTime = 1;
        config.roomID = mRoomID;
        mSDKEngine.startPublishingStream(getMVStreamId(), config, ZegoPublishChannel.AUX);
    }

    //重新开始推伴奏流
    public void restartMvStream() {
        L.i(TAG, "restartMvStream");
        if (mRole == ZGRoomIdentifier.ZGRoomAnchor) {
            startPublishAudio();
            startPublishMV();
            updateMixStream();
        }
    }

    //获得伴奏流的id
    public String getMVStreamId() {
        return String.format("%s%s", mUserInfo.userID, Constants.MV_STREAM_SURFIX);
    }

    //获得台上推流的id
    public String getMainStreamId() {
        return mUserInfo.userID;
    }

    //更新混流
    private void updateMixStream() {
        L.i(TAG, "updateMixStream");
        ZegoMixerTask mixerTask = new ZegoMixerTask(getMixStreamTaskID());
        mixerTask.enableSoundLevel(true);
        mixerTask.setStreamAlignmentMode(ZegoStreamAlignmentMode.TRY);

        ZegoMixerAudioConfig audioConfig = new ZegoMixerAudioConfig();
        audioConfig.bitrate = 128;
        audioConfig.codecID = ZegoAudioCodecID.LOW3;
        audioConfig.channel = ZegoAudioChannel.STEREO;
        mixerTask.setAudioConfig(audioConfig);

        ArrayList<ZegoMixerInput> inputs = new ArrayList<>();
        int length = 0;
        Rect audioRect = new Rect(0, 0, 0, 0);

        ZegoMixerInput publisherInput = new ZegoMixerInput(getMainStreamId(), ZegoMixerInputContentType.AUDIO, audioRect);
        inputs.add(publisherInput);

        ZegoMixerInput publisherMusicInput = new ZegoMixerInput(getMVStreamId(),
                ZegoMixerInputContentType.AUDIO, audioRect);
        inputs.add(publisherMusicInput);

        ZegoMixerInput playStreamInput;
        for (String playStreamID : mUserStreamIDMap.values()) {
            playStreamInput = new ZegoMixerInput(playStreamID, ZegoMixerInputContentType.AUDIO, audioRect);
            length = playStreamID.length();
            if (length >= Constants.SOUND_ID_SURFIX_NUM) {
                playStreamInput.soundLevelID = Integer.parseInt(playStreamID.substring(length - Constants.SOUND_ID_SURFIX_NUM, length));
            }
            inputs.add(playStreamInput);
        }

        mixerTask.setInputList(inputs);

        ZegoMixerOutput output = new ZegoMixerOutput(getMixStreamID());
        ArrayList<ZegoMixerOutput> outputs = new ArrayList<>();
        outputs.add(output);
        mixerTask.setOutputList(outputs);

        mSDKEngine.startMixerTask(mixerTask, new IZegoMixerStartCallback() {
            @Override
            public void onMixerStartResult(int resultCode, JSONObject jsonObject) {
                if (resultCode != 0) {

                }
            }
        });
    }

    // 获得混流的id
    public String getMixStreamID() {
        return mRoomID + Constants.MIX_STREAM_SURFIX;
    }

    // 获得混流的id
    public String getMixStreamTaskID() {
        return Constants.MIX_STREAM_TASK_PREFIX + mRoomID;
    }

    // 停止推流
    public void stopPublishAudio(IRTCCommonCallback callback) {
        L.i(TAG, "stopPublishAudio");
        if (mSDKEngine == null) {
            if (callback != null) {
                callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_NOT_INITIALIZED);
            }
            return;
        }

        mSDKEngine.stopPublishingStream();
        if (callback != null) {
            callback.onRTCCallback(0);
        }
    }

    // 获得ntp时间
    public long getNetworkTime() {
        ZegoNetworkTimeInfo zegoNetworkTimeInfo = mSDKEngine.getNetworkTimeInfo();
        if (zegoNetworkTimeInfo != null) {
            return zegoNetworkTimeInfo.timestamp;
        }
        return 0;
    }

    public void sendSEI(byte[] data) {
        mSDKEngine.sendSEI(data);
    }

    public void setRoomExtraMusic(RoomMusicInfo roomMusicInfo) {
        mSDKEngine.setRoomExtraInfo(mRoomID, "RoomMusicInfo", APIBase.getGson().toJson(roomMusicInfo), null);
        FileDownloadManager.getInstance().downloadFile(roomMusicInfo.getSongID(), roomMusicInfo, true);
    }

    public void setStreamExtraInfo(String extra) {
        mSDKEngine.setStreamExtraInfo(extra, ZegoPublishChannel.MAIN, null);
    }

    //停止拉混流
    public void stopPlayingAux() {
        L.i(TAG, "stopPlayingAux");
        mSDKEngine.stopPlayingStream(getMixStreamID());
    }

    // 开始拉混流
    public void startPlayingAux() {
        //拉混流
        L.i(TAG, "startPlayingAux stream id -> " + getMixStreamID());
        mSDKEngine.setPlayStreamBufferIntervalRange(getMixStreamID(), 500, 3000);
        mSDKEngine.startPlayingStream(getMixStreamID());
    }

    //设置耳返音量
    public void setLoopBackVolume(int volume) {
        this.mHeadphoneMonitorVolumeNum = volume;
        mSDKEngine.setHeadphoneMonitorVolume(volume * 2);
    }

    //设置推流端采集音量
    public void setCaptureVolume(int volume) {
        this.mCaptureVolumeNum = volume;
        mSDKEngine.setCaptureVolume(volume);
    }

    //是否静音（关闭麦克风）[true] 表示静音（关闭麦克风）；[false] 表示开启麦克风。
    public void muteMicrophone(boolean isMuted) {
        mSDKEngine.muteMicrophone(isMuted);
    }

    public int getHeadphoneMonitorVolumeNum() {
        return mHeadphoneMonitorVolumeNum;
    }

    public int getCaptureVolumeNum() {
        return mCaptureVolumeNum;
    }

    public boolean isLoopbackSwitch() {
        return mLoopbackSwitch;
    }

    public int getVerbPresetPos() {
        return mVerbPresetPos;
    }

    public boolean isAnchor() {
        return mRole == ZGRoomIdentifier.ZGRoomAnchor;
    }
}
