package com.zego.ktvchorus.ui;

import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import com.zego.ktvchorus.R;
import com.zego.ktvchorus.model.Constants;
import com.zego.ktvchorus.model.ErrorCodeConstants;
import com.zego.ktvchorus.model.RoomMusicInfo;
import com.zego.ktvchorus.model.TopSongInfo;
import com.zego.ktvchorus.rtc.FileDownloadManager;
import com.zego.ktvchorus.rtc.FileDownloadStateListener;
import com.zego.ktvchorus.rtc.KTVRequestAPI;
import com.zego.ktvchorus.rtc.ZGKTVSongLoadState;
import com.zego.ktvchorus.ui.adapter.EmptyAndLoadMoreAdapterWrapper;
import com.zego.ktvchorus.ui.adapter.SongListContentAdapter;
import com.zego.ktvchorus.ui.custom.KTVLoadMoreView;
import com.zego.ktvchorus.ui.custom.ProcessView;
import com.zego.ktvchorus.utils.ToastUtil;

import java.util.List;

/**
 * 热歌推荐
 */
public class HitSongFragment extends Fragment {

    private View view;
    private RecyclerView mSongListContent;  //歌曲类型列表
    private SongListContentAdapter mSongListContentAdapter;

    private ConstraintLayout mClLoadBtn;
    private TextView mTvReloading;

    private KTVLoadMoreView mLoadMoreView;
    private EmptyAndLoadMoreAdapterWrapper mAdapterWrapper;
    private static final int PAGE_NUM = 10;
    private int pageNum = 1;
    private int mState = Constants.STATE_NORMAL;
    private ProcessView mProcessView;
    private View mSongListView;
    private String topId = "8888";
    private int position = 0;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (view == null) {
            view = inflater.inflate(R.layout.ktv_fragment_song, container, false);
            initView(view);
            initRecycleView();
            loadData();
        }
        return view;
    }

    private void initView(View view) {
        mSongListContent = view.findViewById(R.id.song_list_content);
        mClLoadBtn = view.findViewById(R.id.cl_load_btn);
        mTvReloading = view.findViewById(R.id.tv_reloading);
        mProcessView = new ProcessView(getActivity());
    }

    public void setContext(View mSongListView, int position) {
        this.mSongListView = mSongListView;
        this.position = position;
    }

    @Override
    public void onResume() {
        super.onResume();
        if (mSongListContentAdapter.getmData().size() == 0 && mClLoadBtn.getVisibility() == View.GONE) {
            loading();
        }
    }

    private void initRecycleView() {

        String[] categoryIdList = getResources().getStringArray(R.array.ktv_category_id);
        topId = categoryIdList[position];

        this.mSongListContentAdapter = new SongListContentAdapter(getContext());
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext());
        this.mSongListContent.setLayoutManager(linearLayoutManager);

        //解决刷新闪烁问题
        ((SimpleItemAnimator) this.mSongListContent.getItemAnimator()).setSupportsChangeAnimations(false);

        View emptyView = LayoutInflater.from(getContext()).inflate(R.layout.ktv_search_song_empty, mSongListContent, false);
        mTvReloading.setOnClickListener(v -> {
            loading();
            mState = Constants.STATE_NORMAL;
            loadData();
        });

        mLoadMoreView = new KTVLoadMoreView(getContext());
        mAdapterWrapper = new EmptyAndLoadMoreAdapterWrapper(mSongListContentAdapter);
        mAdapterWrapper.setEmptyView(emptyView);
        mAdapterWrapper.setLoadMoreView(mLoadMoreView);
        this.mSongListContent.setAdapter(this.mAdapterWrapper);

        //加载更多
        mAdapterWrapper.setOnLoadMoreListener(new EmptyAndLoadMoreAdapterWrapper.OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                if (mState == Constants.STATE_FINISH) {
                    mLoadMoreView.showFinish();
                } else {
                    pageNum++;
                    mLoadMoreView.showLoading();
                    loadData();
                }

            }
        });

        mSongListContentAdapter.setOnSongItemClickListener(new SongListContentAdapter.OnSongItemClickListener() {
            @Override
            public void onClick(TopSongInfo.SongsDTO song) {
                RoomMusicInfo roomMusicInfo = new RoomMusicInfo();
                roomMusicInfo.setSongID(song.getSongId());
                roomMusicInfo.setSongName(song.getSongName());
                roomMusicInfo.setSingerName(song.getSingerName());
                roomMusicInfo.setAlbumImg(song.getAlbumImg());

                FileDownloadManager.getInstance().downloadFile(song.getSongId(), roomMusicInfo);
                registerDownloadProgress(song);
            }
        });

    }

    private void registerDownloadProgress(TopSongInfo.SongsDTO bean) {
        String songId = bean.getSongId();
        FileDownloadManager.getInstance().registerDownloadListener(getActivity(), songId, new FileDownloadStateListener() {
            @Override
            public void onDownloadState(ZGKTVSongLoadState state) {
                mSongListContentAdapter.updateDownloadState(songId, state, mSongListContent);
                if (state == ZGKTVSongLoadState.LOAD_COMPLETE) {
                    bean.setLoadState(ZGKTVSongLoadState.LOAD_COMPLETE.value());
                } else if (state == ZGKTVSongLoadState.LOADING) {
                    if (bean.getLoadState() != ZGKTVSongLoadState.LOADING.value()) {
                        bean.setLoadState(ZGKTVSongLoadState.LOADING.value());
                    }
                }
            }

            @Override
            public void onDownloadProgress(float progress) {
                mSongListContentAdapter.updateDownloadProgress(songId, progress, mSongListContent);
            }
        });
    }

    private void loading() {
        mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mSongListView.getRootView());
    }

    /**
     * 加载更多（通过sdk获取列表数据）
     */
    private void loadData() {
        gainSongListCategory(topId, pageNum, PAGE_NUM);
    }

    public void gainSongListCategory(String topId, int page, int size) {
        if (mState == Constants.STATE_NORMAL) {
            KTVRequestAPI.topSong(topId, page, size, (errorCode, topSongInfo) -> {
                if (mProcessView != null) {
                    mProcessView.dismiss();
                }
                if (errorCode == 0) {  //返回成功
                    if (topSongInfo != null && topSongInfo.getSongs() != null) {
                        List<TopSongInfo.SongsDTO> mData = topSongInfo.getSongs();
                        // 返回来的数据不固定，不受size 控制。只有返回的数据为0的时候才不加载下一页
                        if (mData.size() == 0) {
                            mState = Constants.STATE_FINISH;
                            mLoadMoreView.showFinish();
                            loadNoneData();
                        } else {
                            mState = Constants.STATE_NORMAL;
                            mClLoadBtn.setVisibility(View.GONE);
                        }
                        mSongListContentAdapter.addData(topSongInfo.getSongs());
                        mAdapterWrapper.notifyDataSetChanged();
                    } else {
                        mState = Constants.STATE_ERROR;
                        mLoadMoreView.showFinish();
                        loadNoneData();
                    }
                } else {
                    if (errorCode == ErrorCodeConstants.GoChatErrorFailNetwork) {
                        ToastUtil.showWarnToast(R.string.fail_network);
                    }
                    mState = Constants.STATE_ERROR;
                    mLoadMoreView.showFinish();
                    loadNoneData();
                }
            });
        }
    }

    private void loadNoneData() {
        if (pageNum == 1) {
            mClLoadBtn.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        FileDownloadManager.getInstance().unRegisterDownloadListener(getActivity());
    }
}
