package com.zego.ktvchorus.ui;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import com.google.gson.Gson;
import com.zego.ktvchorus.R;
import com.zego.ktvchorus.model.Constants;
import com.zego.ktvchorus.model.HostSEIInfo;
import com.zego.ktvchorus.model.RoomMusicInfo;
import com.zego.ktvchorus.model.StreamExtraInfo;
import com.zego.ktvchorus.model.UserDataMonitorInfo;
import com.zego.ktvchorus.model.UserInfo;
import com.zego.ktvchorus.model.ZGRoomIdentifier;
import com.zego.ktvchorus.model.ZegoDataCenter;
import com.zego.ktvchorus.rtc.APIBase;
import com.zego.ktvchorus.rtc.FileDownloadManager;
import com.zego.ktvchorus.rtc.KTVRoomOrderSongManager;
import com.zego.ktvchorus.rtc.ZGKTVCopyrightedMusicManager;
import com.zego.ktvchorus.rtc.ZGKTVCopyrightedSong;
import com.zego.ktvchorus.rtc.ZGKTVLyricManager;
import com.zego.ktvchorus.rtc.ZGKTVPlayerManager;
import com.zego.ktvchorus.rtc.ZGRealTimeKtvManager;
import com.zego.ktvchorus.rtc.callbacks.IEventCallback;
import com.zego.ktvchorus.rtc.callbacks.IOrderSongListUpdate;
import com.zego.ktvchorus.rtc.callbacks.IZGKTVGetLyricCallback;
import com.zego.ktvchorus.rtc.callbacks.IZGKTVPlayerUpdateListener;
import com.zego.ktvchorus.ui.adapter.KTVRoomStageGuestAdapter;
import com.zego.ktvchorus.ui.custom.AdjustVolumeDialog;
import com.zego.ktvchorus.ui.custom.KTVScoreShowDialog;
import com.zego.ktvchorus.ui.custom.MessageDialog;
import com.zego.ktvchorus.ui.custom.MusicPlayerMenu;
import com.zego.ktvchorus.ui.custom.SongListDialog;
import com.zego.ktvchorus.utils.CheckDoubleClick;
import com.zego.ktvchorus.utils.L;
import com.zego.ktvchorus.utils.PermissionHelper;
import com.zego.ktvchorus.utils.T;

import java.util.List;

import im.zego.lyricview.ZegoLyricView;
import im.zego.lyricview.listener.OnLyricLineFinishedListener;
import im.zego.lyricview.model.LineInfo;
import im.zego.lyricview.model.ZGKTVLyric;
import im.zego.pitchview.ZegoPitchViewHelper;
import im.zego.pitchview.model.ZGKTVPitch;
import im.zego.zegoexpress.ZegoMediaPlayer;
import im.zego.zegoexpress.constants.ZegoMediaPlayerState;

public class KTVRoomActivity extends AppCompatActivity implements IEventCallback {

    private final static String TAG = "KTVRoomActivity";
    private static final int DEF_SEI_COUNT = 7;

    private boolean mIsMicMuted = false;
    private boolean mIsDataMonitorOpen = false;
    private RecyclerView guestRecyclerView;
    private MusicPlayerMenu mMusicPlayerMenu;
    private ZegoLyricView mZegoLyricView;
    private KTVScoreShowDialog ktvScoreShowDialog;

    private KTVRoomStageGuestAdapter stateGuestAdapter;

    int progressInterval = 1;
    private long mPlayerCurrentTimeMillis = 0;//播放器进度回调时的时间，未播放状态需设为0
    private long mPlayerCurrentProgress = 0;//播放器进度，未播放状态需设为0
    private long currentProgress = 0L;
    private int mCurrentPitch = 0;
    private String currentPlaySongId = "";
    private TextView mic;

    private SongListDialog songListDialog;
    private TextView viewNextSong;

    public static void start(Context context) {
        Intent intent = new Intent(context, KTVRoomActivity.class);
        context.startActivity(intent);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_ktv_room);

        initView();
        initMediaPlayer();
        anchorDefaultOpenMic();
        ZGRealTimeKtvManager.getInstance().addListener(this);
    }

    private void initView() {
        mic = findViewById(R.id.mic);
        viewNextSong = findViewById(R.id.next_song);

        mMusicPlayerMenu = findViewById(R.id.music_player_menu);
        mZegoLyricView = mMusicPlayerMenu.getLyricView();

        mZegoLyricView.setHighLightTextSizeSp(20);
        mZegoLyricView.setDefaultTextSizeSp(12);
        mZegoLyricView.setShaderTextSizeSp(12);
        mZegoLyricView.setLineSpaceDp(12);
        mZegoLyricView.setHintTextSizeSp(12);
        // mZegoLyricView.reset(" ");

        guestRecyclerView = findViewById(R.id.rv_guest_list);
        stateGuestAdapter = new KTVRoomStageGuestAdapter(this);
        guestRecyclerView.setLayoutManager(new GridLayoutManager(this, 4));
        guestRecyclerView.setAdapter(stateGuestAdapter);
        ((SimpleItemAnimator) guestRecyclerView.getItemAnimator()).setSupportsChangeAnimations(false);
        orderListUpdate(KTVRoomOrderSongManager.getInstance().getOrderList());
        KTVRoomOrderSongManager.getInstance().addOrderSongListUpdateListener(new IOrderSongListUpdate() {
            @Override
            public void orderSongListUpdate(List<ZGKTVCopyrightedSong> orderList, boolean needSwitchMenu) {
                if (needSwitchMenu) {
                    orderListUpdate(orderList);
                }
            }
        });

        TextView tvRoomId = findViewById(R.id.subject);
        tvRoomId.setText("在线KTV房间 " + ZegoDataCenter.getRoomID());

        hideView();
    }

    private void anchorDefaultOpenMic() {
        micPermission(false);
    }

    private void hideView() {
        if (!ZGRealTimeKtvManager.getInstance().isAnchor()) {
            mic.setVisibility(View.INVISIBLE);
            findViewById(R.id.adjust).setVisibility(View.INVISIBLE);
            viewNextSong.setVisibility(View.INVISIBLE);
            findViewById(R.id.select_song).setVisibility(View.INVISIBLE);
            mMusicPlayerMenu.hideView();
        }
    }

    private void orderListUpdate(List<ZGKTVCopyrightedSong> orderList) {
        if (orderList.size() > 0) {
            if (ZGKTVPlayerManager.getInstance().getTotalScore() > 0) {
                // 已经在唱歌 先展示结算对话框
                showScoreDialog();
            }
            currentPlaySongId = orderList.get(orderList.size() - 1).getSongID();
            switchMusicPlayerMenu(currentPlaySongId);
        }
    }

    private void initMediaPlayer() {
        mZegoLyricView.setOnLyricFinishLineListener(new OnLyricLineFinishedListener() {
            @Override
            public void onLyricLineFinished(int position, LineInfo lineInfo) {
                int lastScore = mMusicPlayerMenu.getScore();
                int totalScore = ZGKTVPlayerManager.getInstance().getTotalScore();
                if (totalScore >= 0) {
                    mMusicPlayerMenu.setScore(totalScore);
                }
                if (mMusicPlayerMenu.getPitchView().getPitchStartTime() < currentProgress) {

                    if (!mIsMicMuted) {
                        int score = ZGKTVPlayerManager.getInstance().getPreviousScore();
                        if (score <= 100) {
                            mMusicPlayerMenu.getPitchView().addScore(score);
                        } else {
                            mMusicPlayerMenu.getPitchView().addScore(totalScore - lastScore);
                        }
                    } else {
                        mMusicPlayerMenu.getPitchView().addScore(totalScore - lastScore);
                    }

                }

            }
        });

        mMusicPlayerMenu.setListener(this::selectSong);
        mMusicPlayerMenu.getSongSwitch().setOnCheckedChangeListener((buttonView, isChecked) -> {
            if (!buttonView.isPressed()) {
                return;
            }
            ZGKTVPlayerManager.getInstance().setAudioOrigin(isChecked);
        });

        ZGKTVPlayerManager.getInstance().setOnIZGKTVPlayerUpdateListener(new IZGKTVPlayerUpdateListener() {
            @Override
            public void onPlaybackStateUpdate(ZegoMediaPlayer mediaPlayer, ZegoMediaPlayerState state, int errorCode) {
                if (errorCode == 0) {
                    if (state != ZegoMediaPlayerState.PLAYING) {
                        mPlayerCurrentTimeMillis = 0;
                        mPlayerCurrentProgress = 0;
                    } else {
                        mPlayerCurrentTimeMillis = System.currentTimeMillis();
                        mPlayerCurrentProgress = mediaPlayer.getCurrentProgress();
                    }
                }

                if (errorCode == 0) {
                    if (state == ZegoMediaPlayerState.PLAYING) {
                        if (!TextUtils.isEmpty(currentPlaySongId)) {
                            switchMusicPlayerMenu(currentPlaySongId);

                            mMusicPlayerMenu.setScore(0);

                            if (mIsMicMuted) {
                                ZGKTVPlayerManager.getInstance().pauseScore();
                                mCurrentPitch = 0;
                            } else {
                                ZGKTVPlayerManager.getInstance().resumeScore();
                            }
                        }

                        ZGKTVPlayerManager.getInstance().getStandardPitch((errorCode1, pitch) -> {
                            if (errorCode1 == 0) {
                                Gson gson = new Gson();
                                Log.d("pengliang", "setStandardPitch");
                                mMusicPlayerMenu.getPitchView().setStandardPitch(ZegoPitchViewHelper.parsePitch(gson.fromJson(pitch, ZGKTVPitch.class)));
                            } else {
                                mMusicPlayerMenu.getPitchView().reset();
                                T.show("获取音高线失败");
                            }
                        });
                    }
                }
                if (state == ZegoMediaPlayerState.PLAY_ENDED) {
                    showScoreDialog();
                    stopCurrentPlayingSong(0);
                }
            }

            @Override
            public void onPlaybackProgressUpdate(ZegoMediaPlayer mediaPlayer, long progress) {
                mPlayerCurrentTimeMillis = System.currentTimeMillis();
                mPlayerCurrentProgress = progress;
                if (ZGRealTimeKtvManager.getInstance().isAnchor()) {
                    long curtime = ZGRealTimeKtvManager.getInstance().getNetworkTime();
                    HostSEIInfo hostSEIInfo = new HostSEIInfo();
                    ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(currentPlaySongId);
                    if (song == null) {
                        return;
                    }
                    hostSEIInfo.setkTotalKey(song.getDuration());
                    hostSEIInfo.setkState(0);
                    hostSEIInfo.setkRole(ZGRoomIdentifier.ZGRoomAnchor.value());
                    hostSEIInfo.setkProgressKey(progress);
                    hostSEIInfo.setkPointTimeKey(curtime);
                    hostSEIInfo.setkResourceID(song.getResourceID());
                    Log.d(TAG, "sendSEI");
                    ZGRealTimeKtvManager.getInstance().sendSEI(APIBase.getGson().toJson(hostSEIInfo).getBytes());
                    updateMusicPlayerMenu();
                }
            }

            @Override
            public void onCurrentPitchValueUpdate(String resourceID, int currentDuration, int pitchValue) {
                if (mIsMicMuted) {
                    return;
                }
                mCurrentPitch = pitchValue;
            }
        });
    }

    private void showScoreDialog() {
        if (!this.isFinishing()) {
            if (ktvScoreShowDialog != null && ktvScoreShowDialog.isShowing()) {
                ktvScoreShowDialog.dismiss();
            }
            ktvScoreShowDialog = new KTVScoreShowDialog(KTVRoomActivity.this);
            int averageScore = ZGKTVPlayerManager.getInstance().getAverageScore();
            int totalScore = ZGKTVPlayerManager.getInstance().getTotalScore();
            ktvScoreShowDialog.setScoreShow(averageScore, totalScore);
            ktvScoreShowDialog.show();
            ZGKTVPlayerManager.getInstance().resetScore();
            mMusicPlayerMenu.setScore(0);
            // mMusicPlayerMenu.orderSong();
        }
    }

    private void updateMusicPlayerMenu() {
        // 计算出当前播放器进度 （当前进度 = 当前时间 - 播放器回调时的时间 + 播放器进度）
        long customProgress = System.currentTimeMillis() - mPlayerCurrentTimeMillis + mPlayerCurrentProgress;
        L.i(TAG, "onPlaybackProgressUpdate:" + customProgress);
        currentProgress = customProgress;
        List<ZGKTVCopyrightedSong> orderList = KTVRoomOrderSongManager.getInstance().getOrderList();
        if (orderList == null || orderList.isEmpty()) {
            return;
        }
        String songId = orderList.get(orderList.size() - 1).getSongID();
        if (songId == null || songId.isEmpty()) {
            return;
        }
        mZegoLyricView.setCurrentTimeMillis(customProgress);

        progressInterval++;
        if (progressInterval >= 2) {
            ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
            long duration = 0;
            if (song != null) {
                duration = song.getDuration();
            }

            if (duration == 0) {
                duration = ZGKTVPlayerManager.getInstance().getTotalDuration();
            }
            mMusicPlayerMenu.updateTime(customProgress, duration);
            progressInterval = 0;
            mMusicPlayerMenu.getPitchView().setCurrentSongProgress(customProgress, mCurrentPitch);
        }
    }

    /**
     * 麦克风按钮点击
     */
    public void switchMic(View view) {
        mIsMicMuted = !mIsMicMuted;
        if (mIsMicMuted) {
            ZGRealTimeKtvManager.getInstance().muteMicrophone(true);
            mic.setCompoundDrawablesWithIntrinsicBounds(null, getDrawable(R.mipmap.ktv_mute_mic), null, null);
            Toast.makeText(KTVRoomActivity.this, "麦克风已关闭", Toast.LENGTH_SHORT).show();
            ZGKTVPlayerManager.getInstance().pauseScore();
            mCurrentPitch = 0;
            stateGuestAdapter.switchMic(false);
        } else {
            // 麦克是关闭状态要开启的时候，需要判断麦克风的权限
            micPermission(true);
        }
    }

    /**
     * 麦克风权限
     */
    private void micPermission(boolean showToast) {
        PermissionHelper.onAudioPermissionGranted(this, allGranted -> {
            if (allGranted) {
                mIsMicMuted = false;
                ZGRealTimeKtvManager.getInstance().muteMicrophone(false);
                mic.setCompoundDrawablesWithIntrinsicBounds(null, getDrawable(R.mipmap.ktv_mic), null, null);
                if (showToast) {
                    Toast.makeText(KTVRoomActivity.this, "麦克风已打开", Toast.LENGTH_SHORT).show();
                }
                ZGKTVPlayerManager.getInstance().resumeScore();
                stateGuestAdapter.switchMic(true);
            } else {
                mIsMicMuted = true;
                MessageDialog messageDialog = new MessageDialog(KTVRoomActivity.this, R.layout.ktv_dialog_msg);
                messageDialog.setMsgTitle("无法使用麦克风");
                messageDialog.setMsgContent("请在系统设置中开启麦克风的使用权限");
                messageDialog.setRightBtnContent("去设置");
                messageDialog.setLeftBtnContent("取消");
                messageDialog.setSureListener(v -> {
                    // 取消
                    messageDialog.dismiss();
                    Toast.makeText(KTVRoomActivity.this, "打开麦克风失败", Toast.LENGTH_SHORT).show();
                });
                messageDialog.setCancelListener(v -> {
                    //去设置
                    messageDialog.dismiss();
                    Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                            .setData(Uri.fromParts("package", getPackageName(), null));
                    startActivityForResult(intent, 666);
                });
            }
        });
    }

    /**
     * 调节菜单点击回调
     *
     * @param view
     */
    public void adjustEffects(View view) {
        AdjustVolumeDialog adjustVolumeDialog = new AdjustVolumeDialog();
        adjustVolumeDialog.show(getSupportFragmentManager(), adjustVolumeDialog.getTag());
    }

    /**
     * 点歌弹窗
     *
     * @param view
     */
    public void selectSong(View view) {
        if (CheckDoubleClick.isFastDoubleClick()) {
            return;
        }

        songListDialog = new SongListDialog(songId -> {
            switchMusicPlayerMenu(songId);
            // 设置房间附加消息
            ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
            RoomMusicInfo roomMusicInfo = new RoomMusicInfo();
            roomMusicInfo.setSongID(song.getSongID());
            roomMusicInfo.setSongName(song.getSongName());
            roomMusicInfo.setSingerName(song.getSingerName());
            roomMusicInfo.setAlbumImg(song.getAlbumImageUri());
            roomMusicInfo.setKrcToken(song.getKrcToken());
            ZGRealTimeKtvManager.getInstance().setRoomExtraMusic(roomMusicInfo);
            stopCurrentPlayingSong(1);

            songListDialog.dismiss();
        });
        songListDialog.show(getSupportFragmentManager(), songListDialog.getTag());
    }

    public void nextSong(View view) {
        if (ZGKTVPlayerManager.getInstance().isPlaying()) {
            stopCurrentPlayingSong(0);
        } else {
            startPlayingSong();
        }
    }

    private void stopCurrentPlayingSong(int stopType) {
        // 停止
        viewNextSong.setText("播放");
        StreamExtraInfo extraInfo = new StreamExtraInfo();
        extraInfo.setMessage("Mediaplayer-Stop");
        extraInfo.setkBehaviorType(Constants.MediaPlayerBehavior.STOP.value());
        extraInfo.setStopType(String.valueOf(stopType));
        ZGRealTimeKtvManager.getInstance().setStreamExtraInfo(APIBase.getGson().toJson(extraInfo));
        if (ZGKTVPlayerManager.getInstance().isPlaying()) {
            showScoreDialog();
        }
        ZGKTVPlayerManager.getInstance().stop();
        if (stopType == 0) {
            mMusicPlayerMenu.orderSong();
        }
    }

    private void startPlayingSong() {
        if (!currentPlaySongId.isEmpty()) {
            // 播放
            viewNextSong.setText("停止");
            StreamExtraInfo extraInfo = new StreamExtraInfo();
            extraInfo.setMessage("Mediaplayer-StartInFuture");
            extraInfo.setkBehaviorType(Constants.MediaPlayerBehavior.PLAYING.value());
            long startTime = ZGRealTimeKtvManager.getInstance().getNetworkTime() + 2000;
            extraInfo.setkStartTime(startTime);

            ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(currentPlaySongId);
            extraInfo.setkResourceID(song.getResourceID());
            ZGKTVPlayerManager.getInstance().startInFuture(startTime, song.getResourceID());

            ZGRealTimeKtvManager.getInstance().setStreamExtraInfo(APIBase.getGson().toJson(extraInfo));
        } else {
            Toast.makeText(KTVRoomActivity.this, "未选择歌曲", Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * 用于房间界面状态切换
     */
    private void switchMusicPlayerMenu(String songId) {
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
        String songName = song == null ? "" : song.getSongName();
        boolean isHasAccompany = song != null && song.isHasAccompany();
        boolean isHasOriginal = song != null && song.isHasOriginal();

        if (ZGRealTimeKtvManager.getInstance().isAnchor()) {
            mMusicPlayerMenu.startPlaying(songName, isHasAccompany, isHasOriginal, true);
        } else {
            mMusicPlayerMenu.startPlaying(songName, isHasAccompany, isHasOriginal, false);
        }
        ZGKTVLyric lrc = ZGKTVLyricManager.getInstance().getLyric(songId);
        if (lrc != null) {
            mMusicPlayerMenu.setLrc(currentPlaySongId, lrc);
        } else {
            ZGKTVCopyrightedMusicManager.getInstance().getKrcLyricForSongID(songId, new IZGKTVGetLyricCallback() {
                @Override
                public void onGetLyricCallback(int errorCode, String message, ZGKTVLyric song) {
                    mMusicPlayerMenu.setLrc(currentPlaySongId, song);
                }
            });
        }

        mMusicPlayerMenu.setSongSwitchVisibility(ZGRealTimeKtvManager.getInstance().isAnchor());
        // mMusicPlayerMenu.getSongSwitch().setChecked(true);
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            return true;
        }
        return false;
    }

    @Override
    protected void onDestroy() {
        ZGRealTimeKtvManager.getInstance().logoutRoom();
        ZGKTVPlayerManager.getInstance().uninit();
        ZGKTVCopyrightedMusicManager.getInstance().clearData();
        ZGKTVCopyrightedMusicManager.getInstance().uninit();
        ZGRealTimeKtvManager.getInstance().uninit();
        KTVRoomOrderSongManager.getInstance().clearOrderList();
        FileDownloadManager.getInstance().clearData();
        super.onDestroy();
    }

    /**
     * 退出房间
     *
     * @param view
     */
    public void exitRoom(View view) {
        ZGRealTimeKtvManager.getInstance().getCaptureVolumeNum();
        finish();
    }

    @Override
    public void onUserListUpdate(List<UserInfo> userList) {
        stateGuestAdapter.onUserListUpdate(userList);
    }

    @Override
    public void onPlayerRecvSEI(String streamId, HostSEIInfo info) {
        if (!ZGRealTimeKtvManager.getInstance().isAnchor()) {
            // 观众同步歌词
            Log.d(TAG, "onPlayerRecvSEI kProgressKey -> " + info.getkProgressKey());
            mZegoLyricView.setCurrentTimeMillis(info.getkProgressKey());
            mMusicPlayerMenu.updateTime(info.getkProgressKey(), info.getkTotalKey());
        }
    }

    @Override
    public void onMusicStop(String stopType) {
        mMusicPlayerMenu.setScore(0);
        viewNextSong.setText("播放");
        if (ZGKTVPlayerManager.getInstance().isPlaying()) {
            showScoreDialog();
        }
        if (stopType.equals("0")) {
            mMusicPlayerMenu.orderSong();
        }
    }

    @Override
    public void onMusicStart() {
        viewNextSong.setText("停止");
    }

    @Override
    public void onPlayQuality(String streamId, UserDataMonitorInfo userDataMonitorInfo) {
        stateGuestAdapter.updateGuestPlayQuality(streamId, userDataMonitorInfo);
    }
}