package com.zego.ktvchorus.ui.adapter;

import android.content.Context;
import android.graphics.drawable.AnimationDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.zego.ktvchorus.R;
import com.zego.ktvchorus.model.Constants;
import com.zego.ktvchorus.model.NetworkQuality;
import com.zego.ktvchorus.model.UserDataMonitorInfo;
import com.zego.ktvchorus.model.UserInfo;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 房间内嘉宾（上麦的人）
 */
public class KTVRoomStageGuestAdapter extends RecyclerView.Adapter<KTVRoomStageGuestAdapter.GuestViewHolder> {
    private ArrayList<UserInfo> guestList = new ArrayList<>(Constants.MAX_STAGE_NUM);
    private ArrayList<UserDataMonitorInfo> mGuestDataList = new ArrayList<>(Constants.MAX_STAGE_NUM);
    private HashMap<String, Float> userVolumeMap = new HashMap<>();
    private OnItemClickListener mOnItemClickListener;
    private AnimationDrawable mAnimationDrawable;
    private boolean mIsShowDataMonitor;
    private Context mContext;

    public KTVRoomStageGuestAdapter(Context context) {
        this.mContext = context;
        mGuestDataList.clear();
        for (int index = 0; index < Constants.MAX_STAGE_NUM; index++) {
            mGuestDataList.add(new UserDataMonitorInfo());
        }
    }

    @NotNull
    @Override
    public GuestViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.ktv_mic_item, parent, false);
        return new GuestViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull GuestViewHolder holder, int position) {
        UserInfo stageGuestUserInfo = guestList.get(position);
        UserDataMonitorInfo userDataMonitorInfo = mGuestDataList.get(position);
        /*if (stageGuestUserInfo.getOnstageState() != 2) {
            userDataMonitorInfo.setNetworkQuality(NetworkQuality.UNKNOWN);
            holder.micIcon.setImageResource(R.mipmap.ktv_on_stage);
            stageGuestUserInfo.setNickName(String.format(Locale.CHINA, "%d号麦", position + 1));
        } else {
        }*/

        Glide.with(mContext)
                .load(stageGuestUserInfo.getAvatar())
                .dontAnimate()
                .circleCrop()
                .diskCacheStrategy(DiskCacheStrategy.ALL) // 设置缓存的策略
                .into(holder.micIcon);

        holder.micTalking.setVisibility(View.VISIBLE);
        holder.networkQuality.setVisibility(View.VISIBLE);

        //        holder.ntpDeviation.setText(String.format(holder.ntpDeviation.getContext().getString(R.string.ktv_ntp_deviation), userDataMonitorInfo.getNtpDeviation()));
        //        holder.p2p.setText(String.format(holder.p2p.getContext().getString(R.string.ktv_p2p), userDataMonitorInfo.getPeerToPeerDelay()));
        //        holder.msd.setText(String.format(holder.msd.getContext().getString(R.string.ktv_msd), userDataMonitorInfo.getMsd()));
        //        holder.packetLoss.setText(String.format(holder.packetLoss.getContext().getString(R.string.ktv_packet_loss), userDataMonitorInfo.getPacketLoss()));
        //        holder.delay.setText(String.format(holder.delay.getContext().getString(R.string.ktv_play_delay), userDataMonitorInfo.getRtt()));

        setNetworkQuality(holder.networkQuality, userDataMonitorInfo.getNetworkQuality());

        if (stageGuestUserInfo.getMic() == 1) {
            holder.muteIcon.setVisibility(View.VISIBLE);
        } else {
            holder.muteIcon.setVisibility(View.GONE);
        }

        if ((stageGuestUserInfo.isStageGuest() || stageGuestUserInfo.isHost()) && mIsShowDataMonitor) {
            holder.dataMonitor.setVisibility(View.VISIBLE);
        } else {
            holder.dataMonitor.setVisibility(View.GONE);
        }

        /*if (userDataMonitorInfo.getPeerToPeerDelay() == 0) {
            holder.ntpDeviation.setVisibility(View.VISIBLE);
            holder.p2p.setVisibility(View.GONE);
            holder.msd.setVisibility(View.GONE);
        } else {
            holder.ntpDeviation.setVisibility(View.GONE);
            holder.p2p.setVisibility(View.VISIBLE);
            holder.msd.setVisibility(View.VISIBLE);
        }*/

        String name = stageGuestUserInfo.getNickName();

        holder.nickName.setText(stageGuestUserInfo.getNickName());
        holder.micIcon.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mOnItemClickListener != null) {
                    mOnItemClickListener.onItemClick(holder.getAdapterPosition(), stageGuestUserInfo);
                }
            }
        });
        mAnimationDrawable = (AnimationDrawable) holder.micTalking.getBackground();
        Float volume = findSoundLevelById(stageGuestUserInfo.getUserID());
        if (volume != null && volume > 1 && stageGuestUserInfo.isMicOpen()) {
            if (!mAnimationDrawable.isRunning()) {
                mAnimationDrawable.start();
            }
        } else {
            mAnimationDrawable.stop();
            mAnimationDrawable.selectDrawable(0);
        }
    }

    private Float findSoundLevelById(long uid) {
        String uidS = String.valueOf(uid);
        if (uidS.length() < 5) {
            return null;
        }
        String surfix = null;
        if (uidS.length() >= Constants.SOUND_ID_SURFIX_NUM) {
            surfix = uidS.substring(uidS.length() - Constants.SOUND_ID_SURFIX_NUM, uidS.length());
        }
        if (userVolumeMap.containsKey(surfix)) {
            return userVolumeMap.get(surfix);
        } else if (userVolumeMap.containsKey(uidS)) {
            return userVolumeMap.get(uidS);
        }
        return null;

    }

    public void setOnclickItemListner(OnItemClickListener listner) {
        mOnItemClickListener = listner;
    }

    @Override
    public int getItemCount() {
        return guestList.size();
    }

    public void switchMic(boolean open) {
        if (guestList.size() > 0) {
            UserInfo stageGuestUserInfo = guestList.get(0);
            stageGuestUserInfo.setMicOpen(open);
            notifyDataSetChanged();
        }
    }

    /**
     * 更新网络数据
     *
     * @param streamId
     * @param userDataMonitorInfo
     */
    public void updateGuestPlayQuality(String streamId, UserDataMonitorInfo userDataMonitorInfo) {
        for (int index = 0; index < guestList.size(); index++) {
            if (streamId.equals(String.valueOf(guestList.get(index).getUserID()))) {
                // userDataMonitorInfo.setMsd(mGuestDataList.get(index).getMsd());
                mGuestDataList.set(index, userDataMonitorInfo);
                notifyItemChanged(index);
                break;
            }
        }
    }

    /**
     * 更新网络状态
     *
     * @param quality
     */
    public void updateGuestPlayQuality(NetworkQuality quality) {
        for (int index = 0; index < mGuestDataList.size(); index++) {
            UserDataMonitorInfo userDataMonitorInfo = mGuestDataList.get(index);
            userDataMonitorInfo.setNetworkQuality(quality);
            mGuestDataList.set(index, userDataMonitorInfo);
            notifyItemChanged(index);
        }
    }

    /**
     * 更新网络数据
     *
     * @param uid
     * @param msd
     */
    public void updateGuestMsd(long uid, long msd) {
        for (int index = 0; index < guestList.size(); index++) {
            if (uid == guestList.get(index).getUserID()) {
                UserDataMonitorInfo userDataMonitorInfo = mGuestDataList.get(index);
                userDataMonitorInfo.setMsd(msd);
                mGuestDataList.set(index, userDataMonitorInfo);
            }
        }
    }


    /**
     * 初始化嘉宾列表
     */
    public void initUserList() {
        guestList.clear();
        for (int index = 0; index < Constants.MAX_STAGE_NUM; index++) {
            guestList.add(new UserInfo());
        }
        notifyDataSetChanged();
    }

    /**
     * 根据声音的回调播放声音波浪
     *
     * @param userVolumeMap
     */
    public void onUserVolumeUpdated(HashMap<String, Float> userVolumeMap) {
        this.userVolumeMap.putAll(userVolumeMap);
        for (String uid : userVolumeMap.keySet()) {
            int idx = indexOfStringUser(uid);
            if (idx >= 0) {
                if (guestList.get(idx).isMicOpen()) {
                    notifyItemChanged(idx);
                }
            }
        }
    }

    /**
     * 根据id获取当前的麦位位置
     *
     * @param uid
     * @return
     */
    private int indexOfStringUser(String uid) {
        for (int index = 0; index < guestList.size(); index++) {
            if (String.valueOf(guestList.get(index).getUserID()).endsWith(uid)) {
                return index;
            }
        }
        return -1;
    }

    public void onUserListUpdate(List<UserInfo> userList) {
        guestList = (ArrayList<UserInfo>) userList;
        notifyDataSetChanged();
    }

    /**
     * 展示网络数据的按钮切换
     *
     * @param isDataMonitorOpen
     */
    public void setDataMonitorShowStatus(boolean isDataMonitorOpen) {
        if (mIsShowDataMonitor != isDataMonitorOpen) {
            mIsShowDataMonitor = isDataMonitorOpen;
            notifyDataSetChanged();
        }
    }

    static class GuestViewHolder extends RecyclerView.ViewHolder {

        private ImageView micIcon;
        private ImageView micTalking;
        private ImageView muteIcon;
        private LinearLayout dataMonitor;
        private TextView ntpDeviation;
        private TextView p2p;
        private TextView msd;
        private TextView packetLoss;
        private TextView delay;
        private TextView nickName;
        private ImageView networkQuality;

        public GuestViewHolder(View itemView) {
            super(itemView);
            micIcon = itemView.findViewById(R.id.mic_icon);
            micTalking = itemView.findViewById(R.id.mic_talking);
            muteIcon = itemView.findViewById(R.id.mute_icon);
            dataMonitor = itemView.findViewById(R.id.data_monitor);
            ntpDeviation = itemView.findViewById(R.id.ntp_deviation);
            p2p = itemView.findViewById(R.id.p2p);
            msd = itemView.findViewById(R.id.msd);
            packetLoss = itemView.findViewById(R.id.packet_loss);
            delay = itemView.findViewById(R.id.delay);
            nickName = itemView.findViewById(R.id.nick_name);
            networkQuality = itemView.findViewById(R.id.network_quality);
        }

    }

    public interface OnItemClickListener {
        void onItemClick(int position, UserInfo info);
    }

    public static void setNetworkQuality(ImageView view, NetworkQuality quality) {
        if (quality == null) {
            return;
        }
        switch (quality) {
            case BAD:
                view.setImageResource(R.mipmap.ktv_bad_network);
                break;
            case FINE:
                view.setImageResource(R.mipmap.ktv_fine_network);
                break;
            case GRATE:
                view.setImageResource(R.mipmap.ktv_great_network);
                break;
            default:
                break;
        }
    }
}
