package com.zego.ktvchorus.ui.adapter;


import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.zego.ktvchorus.R;
import com.zego.ktvchorus.rtc.ZGKTVCopyrightedSong;

import java.util.ArrayList;
import java.util.List;

public class WaitListAdapter extends RecyclerView.Adapter<WaitListAdapter.MyViewHolder> {
    private List<ZGKTVCopyrightedSong> data = new ArrayList<>();
    private Context context;
    private OnItemSelectListener onItemSelectListener;

    public WaitListAdapter(Context context) {
        this.context = context;
    }

    @NonNull
    @Override
    public MyViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.ktv_item_choose_song, parent, false);

        return new MyViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull MyViewHolder holder, int position) {
        if (data.size() <= position) {
            return;
        }
        ZGKTVCopyrightedSong info = data.get(position);

        Glide.with(context)
                .load(info.getAlbumImageUri())
                .dontAnimate()
                .circleCrop()
                .diskCacheStrategy(DiskCacheStrategy.ALL) // 设置缓存的策略
                .into(holder.mSongCover);

        holder.mSongName.setText(info.getSongName());
        holder.mSongWriter.setText(info.getSingerName());
        holder.mSongSelect.setOnClickListener((buttonView) -> {
            onItemSelectListener.onMusicSelected(info.getSongID());
        });
    }

    @Override
    public int getItemCount() {
        return data.size();
    }

    public void setData(List<ZGKTVCopyrightedSong> data) {
        this.data = data;
    }

    static class MyViewHolder extends RecyclerView.ViewHolder {
        private TextView mSongName;
        private TextView mSongWriter;
        private ImageView mSongCover;
        private TextView mSongSelect;

        public MyViewHolder(@NonNull View itemView) {
            super(itemView);
            mSongName = itemView.findViewById(R.id.song_name);
            mSongCover = itemView.findViewById(R.id.song_cover);
            mSongWriter = itemView.findViewById(R.id.song_writer);
            mSongSelect = itemView.findViewById(R.id.tv_select);
        }
    }

    public void setOnItemSelectListener(OnItemSelectListener onItemSelectListener) {
        this.onItemSelectListener = onItemSelectListener;
    }

    public interface OnItemSelectListener {
        void onMusicSelected(String songId);
    }
}