package com.zego.ktvchorus.ui.custom;

import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import com.zego.ktvchorus.R;
import com.zego.ktvchorus.rtc.KTVRoomOrderSongManager;
import com.zego.ktvchorus.rtc.ZGKTVCopyrightedSong;
import com.zego.ktvchorus.ui.adapter.WaitListAdapter;

import java.util.List;

/**
 * 选择歌曲弹窗（点歌/已点）
 */
public class SongListDialog extends BaseBottomSheetDialogFragment implements View.OnClickListener {

    // 点歌
    private SongListView mSongListView;
    private TextView mSongList;
    // 已点
    private WaitListView mWaitListView;
    private WaitListAdapter.OnItemSelectListener listener;
    private TextView mWaitList;
    private View mVSongList, mVWaitList;

    @Override
    public int getLayoutResId() {
        return R.layout.ktv_dialog_song;
    }

    public SongListDialog(WaitListAdapter.OnItemSelectListener listener) {
        super();
        this.listener = listener;
    }

    @Override
    public void initView(Bundle savedInstanceState) {

        setHeight(450);
        setShowHeight(true); //设置弹窗最大高度

        mSongListView = rootView.findViewById(R.id.song_list_view);
        mWaitListView = rootView.findViewById(R.id.wait_list_view);
        mWaitListView.setOnItemSelectListener(listener);
        mSongList = rootView.findViewById(R.id.song_list);
        mWaitList = rootView.findViewById(R.id.wait_list);
        mVSongList = rootView.findViewById(R.id.v_song_list);
        mVWaitList = rootView.findViewById(R.id.v_wait_list);

        mSongList.setOnClickListener(this);
        mWaitList.setOnClickListener(this);

        selectView(mSongList);
        unSelectView(mWaitList);

        mSongListView.setContext(getActivity());
        initData();
    }

    private void initData() {
        updateOrderSongList(KTVRoomOrderSongManager.getInstance().getOrderList());
        KTVRoomOrderSongManager.getInstance().addOrderSongListUpdateListener(
                (orderList, needSwitchMenu) -> updateOrderSongList(orderList));
    }

    private void updateOrderSongList(List<ZGKTVCopyrightedSong> orderList) {
        String orderStr = "0";
        if (orderList != null) {
            if (orderList.size() > 99) {
                orderStr = "99+";
            } else {
                orderStr = String.valueOf(orderList.size());
            }
        }
        mWaitList.setText("已点(" + orderStr + ")");
        mWaitListView.refresh();
    }

    /**
     * 选中效果
     *
     * @param view
     */
    private void selectView(TextView view) {
        view.setTextColor(getContext().getResources().getColor(R.color.ktv_color_white));
    }

    /**
     * 未选中效果
     *
     * @param view
     */
    private void unSelectView(TextView view) {
        view.setTextColor(getContext().getResources().getColor(R.color.ktv_color_white_cc));
    }


    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.song_list) {
            selectView(mSongList);
            unSelectView(mWaitList);
            mSongListView.setVisibility(View.VISIBLE);
            mVSongList.setVisibility(View.VISIBLE);
            mWaitListView.setVisibility(View.GONE);
            mVWaitList.setVisibility(View.GONE);
        } else if (id == R.id.wait_list) {
            selectView(mWaitList);
            unSelectView(mSongList);
            mSongListView.setVisibility(View.GONE);
            mWaitListView.setVisibility(View.VISIBLE);
            mVSongList.setVisibility(View.GONE);
            mVWaitList.setVisibility(View.VISIBLE);
        }
    }
}