package com.zego.ktvchorus.ui.custom;

import android.content.Context;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.RecyclerView;

import com.zego.ktvchorus.R;
import com.zego.ktvchorus.rtc.KTVRoomOrderSongManager;
import com.zego.ktvchorus.ui.adapter.EmptyAndLoadMoreAdapterWrapper;
import com.zego.ktvchorus.ui.adapter.WaitListAdapter;

/**
 * 已点列表
 */
public class WaitListView extends ConstraintLayout {

    private RecyclerView mWaitList;

    private WaitListAdapter mAdapter;
    private EmptyAndLoadMoreAdapterWrapper mAdapterWrapper;

    public WaitListView(@NonNull Context context) {
        this(context, null);
    }

    public WaitListView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public WaitListView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        View view = View.inflate(context, R.layout.ktv_view_wait_list, this);
        mWaitList = view.findViewById(R.id.wait_list);

        initRecycleView();
        initData();
    }

    private void initRecycleView() {
        mAdapter = new WaitListAdapter(getContext());

        View emptyView = LayoutInflater.from(getContext()).inflate(R.layout.ktv_view_ordered_empty_list, mWaitList, false);
        View view = new View(getContext());
        mAdapterWrapper = new EmptyAndLoadMoreAdapterWrapper(mAdapter);
        mAdapterWrapper.setEmptyView(emptyView);
        mAdapterWrapper.setLoadMoreView(view);
        mWaitList.setAdapter(mAdapterWrapper);
    }

    private void initData() {
        refresh();
    }

    public void refresh() {
        mAdapter.setData(KTVRoomOrderSongManager.getInstance().getOrderList());
        mAdapterWrapper.notifyDataSetChanged();
    }

    public void setOnItemSelectListener(WaitListAdapter.OnItemSelectListener listener) {
        mAdapter.setOnItemSelectListener(listener);
    }
}
