package com.zego.ktvchorus.utils;

import android.content.Context;
import android.widget.ImageView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.zego.ktvchorus.BaseApplication;

public class GlideImageLoader {

    /**
     * 单例
     */
    private static volatile GlideImageLoader mInstance;

    private GlideImageLoader() {
    }

    public static GlideImageLoader getInstance() {
        if (mInstance == null) {
            synchronized (GlideImageLoader.class) {
                if (mInstance == null) {
                    mInstance = new GlideImageLoader();
                    return mInstance;
                }
            }
        }
        return mInstance;
    }

    public int getDrawableIdByTimestamp(long timestamp) {
        String name = "ktv_user" + timestamp % 24;
        Context context = BaseApplication.sApplication;
        return context.getResources().getIdentifier(name, "mipmap", context.getPackageName());
    }

    /**
     * 在一个imageView里面异步展示一个图片
     *
     * @param imageView
     */
    public void displayAvatar(long timestamp, ImageView imageView) {
        int resourceId = getDrawableIdByTimestamp(timestamp);
        Glide.with(imageView.getContext())
                .load(resourceId)
                .dontAnimate()
                .circleCrop()
                .diskCacheStrategy(DiskCacheStrategy.NONE) // 设置缓存的策略
                .into(imageView);
    }
}
